@@ -2,6 +2,7 @@ import collections
 import datetime
 import functools
 import uuid
+import warnings
 
 import yaml
 
@@ -38,31 +39,82 @@ class Projects:
         return Project(project_id=project_id, config=config, **kwargs)
 
 
+def prepare_config(config, namespace=None, role_arn=None, region_name=None):
+    """
+    Prepare the config.  Fill in overrides or defaults as necessary.
+    """
+    # We always want a namespace to be set.  Read it from the initial config
+    # if possible, or use the override or default if not.
+    if namespace and ("namespace" in config) and (config["namespace"] != namespace):
+        warnings.warn(
+            f"Preferring override namespace {namespace} "
+            f"to namespace in config {config['namespace']}"
+        )
+        config["namespace"] = namespace
+    elif "namespace" not in config:
+        config["namespace"] = namespace or DEFAULT_ECR_NAMESPACE
+
+    assert "namespace" in config
+
+    # We always want a role_arn to be set.  Read it from the initial config,
+    # or raise an error if not -- there's no way to pick a sensible default.
+    if role_arn:
+        if ("role_arn" in config) and (config["role_arn"] != role_arn):
+            warnings.warn(
+                f"Preferring override role_arn {role_arn} "
+                f"to role_arn in config {config['role_arn']}"
+            )
+            config["role_arn"] = role_arn
+        elif "role_arn" not in config:
+            config["role_arn"] = role_arn
+
+    if "role_arn" not in config:
+        raise ConfigError("role_arn is not set!")
+
+    assert "role_arn" in config
+
+    # We always want a region_name to be set.  Read it from the initial config
+    # if possible, or use the override or default if not.
+    if region_name and ("region_name" in config) and (config["region_name"] != region_name):
+        warnings.warn(
+            f"Preferring override region_name {region_name} "
+            f"to region_name in config {config['region_name']}"
+        )
+        config["region_name"] = region_name
+    elif "region_name" not in config:
+        config["region_name"] = region_name or DEFAULT_REGION_NAME
+
+    assert "region_name" in config
+
+    # The image repositories are stored as a list of dicts:
+    #
+    #     [
+    #       {"id": "worker1", "services": […]},
+    #       {"id": "worker2", "services": […]},
+    #       ...
+    #     ]
+    #
+    # We don't want to change the structure, but we do want to check that IDs
+    # are unique.
+    repo_id_tally = collections.Counter()
+    for repo in config.get("image_repositories", []):
+        repo_id_tally[repo["id"]] += 1
+
+    duplicates = {repo_id for repo_id, count in repo_id_tally.items() if count > 1}
+    if duplicates:
+        raise ConfigError(
+            f"Duplicate repo{'s' if len(duplicates) > 1 else ''} "
+            f"in image_repositories: {', '.join(sorted(duplicates))}"
+        )
+
+
 class Project:
-    def __init__(self, project_id, config, region_name=None, role_arn=None, account_id=None, namespace=None):
-        self.id = project_id
+    def __init__(self, project_id, config, account_id=None, **kwargs):
+        prepare_config(config, **kwargs)
         self.config = config
 
         self.config['id'] = project_id
 
-        if namespace:
-            self.config['namespace'] = namespace
-        else:
-            if 'namespace' not in self.config:
-                self.config['namespace'] = DEFAULT_ECR_NAMESPACE
-
-        if role_arn:
-            self.config['role_arn'] = role_arn
-        else:
-            if 'role_arn' not in self.config:
-                raise ConfigError("role_arn is not set!")
-
-        if region_name:
-            self.config['region_name'] = region_name
-        else:
-            if 'region_name' not in self.config:
-                self.config['region_name'] = DEFAULT_REGION_NAME
-
         iam = Iam(
             self.config['role_arn'],
             self.config['region_name']
@@ -79,12 +131,7 @@ class Project:
             if 'account_id' not in self.config:
                 self.config['account_id'] = self.user_details['caller_identity']['account_id']
 
-        # Initialise project level vars
-        self.role_arn = self.config['role_arn']
-        self.region_name = self.config['region_name']
-        self.account_id = self.config['account_id']
-        self.namespace = self.config['namespace']
-        self.image_repositories = self.config.get('image_repositories', [])
+        assert "account_id" in self.config
 
         # Create required services
         self.releases_store = DynamoDbReleaseStore(
@@ -93,11 +140,61 @@ class Project:
             role_arn=self.role_arn
         )
 
-        self.prepared_releases = {}
-
         # Ensure release store is available
         self.releases_store.initialise()
 
+    @property
+    def id(self):
+        return self.config["id"]
+
+    @property
+    def role_arn(self):
+        return self.config["role_arn"]
+
+    @property
+    def region_name(self):
+        return self.config["region_name"]
+
+    @property
+    def account_id(self):
+        return self.config["account_id"]
+
+    @property
+    def namespace(self):
+        return self.config["namespace"]
+
+    @property
+    def image_repositories(self):
+        """
+        Gets all the image repositories in the config, keyed by ID.
+
+        Every repository will have the following keys:
+
+        -   account_id
+        -   region_name
+        -   role_arn
+        -   repository_name
+
+        """
+        result = {}
+
+        for repo in self.config.get("image_repositories", []):
+            namespace = repo.get("namespace", self.namespace)
+
+            # We should have uniqueness by the checks in prepare_config(), but
+            # it doesn't hurt to check.
+            assert repo["id"] not in result, repo["id"]
+
+            result[repo["id"]] = {
+                "account_id": repo.get("account_id", self.account_id),
+                "region_name": repo.get("region_name", self.region_name),
+                "role_arn": repo.get("role_arn", self.role_arn),
+                "repository_name": f"{namespace}/{repo['id']}"
+            }
+
+        assert len(result) == len(self.config.get("image_repositories", []))
+        return result
+
     def _ecr(self, account_id=None, region_name=None, role_arn=None):
         return Ecr(
             account_id=account_id or self.account_id,
@@ -135,17 +232,6 @@ class Project:
             "deployments": []
         }
 
-    def _match_image_id(self, image_id):
-        matched_images = [image for image in self.image_repositories if image['id'] == image_id]
-
-        if len(matched_images) > 1:
-            raise RuntimeError(f"Multiple matching images found for {image_id}: ({matched_images}!")
-
-        if matched_images:
-            return matched_images[0]
-        else:
-            return None
-
     def get_environment(self, environment_id):
         environments = {
             e['id']: e for e in self.config.get('environments', []) if 'id' in e
@@ -184,11 +270,9 @@ class Project:
         Generates a set of tuples (image_id, List[services])
         """
         for image_id, _ in release["images"].items():
-            # TODO: self.image_repositories should be a dict, not a list
-            matched_image = self._match_image_id(image_id)
-
-            # TODO: Should this ever happen?
-            if matched_image is None:
+            try:
+                matched_image = self.image_repositories[image_id]
+            except KeyError:
                 continue
 
             try:
@@ -243,15 +327,17 @@ class Project:
         matched_services = {}
         for image_id, _ in release['images'].items():
             # Attempt to match deployment image id to config and override service_ids
-            matched_image = self._match_image_id(image_id)
+            try:
+                matched_image = self.image_repositories[image_id]
+            except KeyError:
+                continue
 
             available_services = []
 
-            if matched_image:
-                services = matched_image.get('services', [])
+            services = matched_image.get('services', [])
 
-                available_services = [_get_service(service) for service in services]
-                available_services = [service for service in available_services if service["response"]]
+            available_services = [_get_service(service) for service in services]
+            available_services = [service for service in available_services if service["response"]]
 
             if available_services:
                 matched_services[image_id] = available_services
@@ -313,30 +399,20 @@ class Project:
 
     def publish(self, image_id, label):
         # Attempt to match image to config
-        matched_image = self._match_image_id(image_id)
-
-        # Assume default account id & namespace
-        account_id = self.account_id
-        namespace = self.namespace
-
-        # If we find a match, set overrides
-        if matched_image:
-            # Check if namespace/account_id is overridden for this image
-            namespace = matched_image.get('namespace', self.namespace)
-            account_id = matched_image.get('account_id', self.account_id)
+        matched_image = self.image_repositories[image_id]
 
         # Create an ECR client for the correct account
-        ecr = self._ecr(account_id)
+        ecr = self._ecr(matched_image["account_id"])
 
         ecr.login()
 
         remote_uri, remote_tag, local_tag = ecr.publish_image(
-            namespace=namespace,
+            namespace=matched_image["namespace"],
             image_id=image_id,
         )
 
         tag_result = ecr.tag_image(
-            namespace=namespace,
+            namespace=matched_image["namespace"],
             image_id=image_id,
             tag=remote_tag,
             new_tag=label
@@ -352,20 +428,8 @@ class Project:
         }
 
     def get_images(self, from_label):
-        image_repositories = {}
-
-        for repo in self.image_repositories:
-            namespace = repo.get("namespace", self.namespace)
-
-            image_repositories[repo["id"]] = {
-                "account_id": repo.get("account_id", self.account_id),
-                "region_name": repo.get("region_name", self.region_name),
-                "role_arn": repo.get("role_arn", self.role_arn),
-                "repository_name": f"{namespace}/{repo['id']}"
-            }
-
         return ecr.get_ref_tags_for_repositories(
-            image_repositories=image_repositories,
+            image_repositories=self.image_repositories,
             tag=from_label
         )
 
@@ -428,14 +492,16 @@ class Project:
         old_tag = image_name.split(":")[-1]
         new_tag = f"env.{environment_id}"
 
-        matched_image = self._match_image_id(image_id)
-        if matched_image:
+        try:
+            matched_image = self.image_repositories[image_id]
             ecr = self._ecr(
-                account_id=matched_image.get('account_id'),
-                region_name=matched_image.get('region_name'),
-                role_arn=matched_image.get('role_arn'),
+                account_id=matched_image["account_id"],
+                region_name=matched_image["region_name"],
+                role_arn=matched_image["role_arn"],
             )
-        else:
+        except KeyError:
+            # TODO: Does it make sense to create an ECR client if we don't
+            # have an ECR repo to tag in?
             ecr = self._ecr()
 
         return ecr.tag_image(
